//******************************************************************************
//! @file $RCSfile: SCCB.h,v $
//!
//! @version $Revision: 1.00 $ $Name: Dewei(dewei(at)mycool(dot)eu)$
//!	
//!	Compiler: avr-gcc 4.3.0 / avr-libc 1.6.2
//!
//! @brief: Driver for SCCB(Serial Camera Control Bus)
//!         
//!
//! Copyright (c) 2010 Dewei Huang 	All rights reserved.
//!
//! This file can be parsed by Doxygen for automatic documentation generation.
//! This file has been validated with AVRStudio-4.16.628 /WinAVR-20080610.
//!
//!
//! @todo:
//! @bug:
//******************************************************************************

//_____  I N C L U D E S _______________________________________________________

#include <util/delay.h>

#include "SCCB.h"


//_____ F U N C T I O N S ______________________________________________________

//------------------------------------------------------------------------------
//  @fn InitSCCB
//!
//! @brief: initialization of SCCB interface
//!         
//!
//! @warning: none
//!
//! @param none
//!
//! @return none
//!
//------------------------------------------------------------------------------
void InitSCCB(void)
{
	SCCB_DDR |= (1 << SCCB_SIO_C) | (1 << SCCB_SIO_D);
	SCCB_PORT |= (1 << SCCB_SIO_C) | (1 << SCCB_SIO_D);
}

//------------------------------------------------------------------------------
//  @fn startSCCB
//!
//! @brief: Starts communication with SCCB interface
//!         
//!
//! @warning: none
//!
//! @param none
//!
//! @return none
//!
//------------------------------------------------------------------------------
void startSCCB(void)
{
	SIO_D_SET;     	// sets data line as high
    _delay_us(100);

    SIO_C_SET;	   	// sets clock line as high
    _delay_us(100);
 
    SIO_D_CLR;		// sets data line as low by clock high
    _delay_us(100);

    SIO_C_CLR;	 	// clear clock line
    _delay_us(100);
}

//------------------------------------------------------------------------------
//  @fn stopSCCB
//!
//! @brief: Stops communication with SCCB interface
//!         
//!
//! @warning: none
//!
//! @param none
//!
//! @return none
//!
//------------------------------------------------------------------------------
void stopSCCB(void)
{
	SIO_D_CLR;
    _delay_us(100);
 
    SIO_C_SET;
    _delay_us(100);
  

    SIO_D_SET;
    _delay_us(100);
	//SCCB_DDR &= ~(SCCB_SIO_D);

}


//------------------------------------------------------------------------------
//  @fn noAck
//!
//! @brief: ready to stop communication with SCCB interface 
//!         
//!
//! @warning: none
//!
//! @param none
//!
//! @return none
//!
//------------------------------------------------------------------------------
void noAck(void)
{
	SIO_D_SET;
	_delay_us(100);
	
	SIO_C_SET;
	_delay_us(100);
	
	SIO_C_CLR;
	_delay_us(100);
	
	SIO_D_CLR;
	_delay_us(100);
}


//------------------------------------------------------------------------------
//  @fn SCCBwriteByte
//!
//! @brief: Write a byte data to SCCB interface
//!         
//!
//! @warning: none
//!
//! @param m_data: data will be writed to SCCB
//!
//! @return integer o: write error
//!					1: write success
//!
//------------------------------------------------------------------------------
unsigned char SCCBwriteByte(unsigned char m_data)
{
	unsigned char j,tem;

	for(j = 0; j < 8; j++) 
	{
		if((m_data << j) & 0x80)
		{
			SIO_D_SET;
		}
		else
		{
			SIO_D_CLR;
		}
		_delay_us(100);
		SIO_C_SET;
		_delay_us(100);
		SIO_C_CLR;
		_delay_us(100);
	}
	_delay_us(100);
	
	SIO_D_IN; 		// sets SDA as input
	_delay_us(100);
	SIO_C_SET;
	_delay_us(1000);
	if(SIO_D_STATE)
	{
		tem = 0;   	// SDA=1, send error
	}
	else
	{
		tem = 1;   	// SDA=0, send success
	}
	
	SIO_C_CLR;
	_delay_us(100);	
    SIO_D_OUT;		// sets SDA as output

	return(tem);  
}


//------------------------------------------------------------------------------
//  @fn SCCBreadByte
//!
//! @brief: Reads a byte data from SCCB bus
//!         
//!
//! @warning: none
//!
//! @param none
//!
//! @return unsigned char: a byte data
//!
//------------------------------------------------------------------------------
unsigned char SCCBreadByte(void)
{
	unsigned char read,j;
	read = 0x00;
	
	SIO_D_IN;			// Sets SDA as input
	_delay_us(100);
	for(j = 8;j > 0; j--) 	
	{		     
		_delay_us(100);
		SIO_C_SET;
		_delay_us(100);
		read = read << 1;
		if(SIO_D_STATE) 
		{
			read = read + 1;
		}
		SIO_C_CLR;
		_delay_us(100);
	}	
	return(read);
}

