// touch hardware config
// TP_CS    PB4
// TP_INT   PD3    INT1
// SCK      PB7    SPI_SCK
// MOSI     PB5    SPI_MOSI
// MISO     PB6    SPI_MISO

/*
7  6 - 4  3      2     1-0
s  A2-A0 MODE SER/DFR PD1-PD0
*/
#define TOUCH_MSR_Y  0xD0   // read X koordination 
#define TOUCH_MSR_X  0x90   // read Y koordination 

//-----------------------------------------------------
#include <avr/io.h>
#include <avr/interrupt.h>

#define SPI_CS_Assert()     PORTB &= ~(1<<PB4)
#define SPI_CS_Deassert()   PORTB |=  (1<<PB4)
//-----------------------------------------------------

//spi low speed
void SPI_Low(void)
{
    SPCR =   _BV(SPE)|_BV(MSTR)|_BV(SPR1)|_BV(SPR0);
    SPSR &= ~_BV(SPI2X);
}

//spi full speed
void SPI_High(void)
{
    SPCR =  _BV(SPE)|_BV(MSTR);
    SPSR |= _BV(SPI2X);
}

//port initialize
void SPI_Init(void)
{
    // sck and mosi port init: output
    DDRB  |= (1<<PB5) | (1<<PB7);
    // miso port init: input
    DDRB &= ~(1<<PB6);
    // enable pup
    PORTB |= 1<<6;
    SPI_Low();
}

//read and write one byte
uint8_t SPI_WriteByte(uint8_t val)
{
    SPDR = val;
    while(!(SPSR & _BV(SPIF)));
    return SPDR;
}

void Init_INT0(void)
{
    SREG  |= 0x80 ; // enable global interrupt
    MCUCR &= ~( (1<<ISC10) | (1<<ISC11) ); // interrupt by positive edge
    GICR  |= (1<<INT1) ;
}

void touch_init(void)
{
    // CS PORT init
    DDRB |= 1<<PB4;
    SPI_CS_Deassert();

    SPI_Init();

    DDRD &= ~(1<<PD3);
    Init_INT0();

    // enable touch int
    SPI_WriteByte(0<<7);
}

#include "ili_lcd_general.h"
#include "lcd_api.h"
#include <stdio.h>

char str_buf[100];
volatile uint8_t touch_int_flag = 0;

void show_toch(void)
{
    uint16_t x,y;

    SPI_CS_Assert();
    SPI_WriteByte(TOUCH_MSR_X);                      /* read X */
    x = SPI_WriteByte(0x00)<<4;                      /* read MSB bit[11:8] */
    x |= ((SPI_WriteByte(TOUCH_MSR_Y)>>4)&0x0F );    /* read LSB bit[7:0] */
    y = SPI_WriteByte(0x00)<<4;                      /* read MSB bit[11:8] */
    y |= ((SPI_WriteByte(0x00)>>4)&0x0F );           /* read LSB bit[7:0] */

    //
    LCD_write_english_string(0,0,"touch:",RED,BLACK);
    sprintf(str_buf,"read X:%04d",x);
    LCD_write_english_string(0,20,str_buf,GREEN,BLACK);
    sprintf(str_buf,"read Y:%04d",y);
    LCD_write_english_string(0,40,str_buf,GREEN,BLACK);

    SPI_WriteByte( 0<<7 ); /* enable interrupt */
    SPI_CS_Deassert();

    // enable int
    touch_int_flag = 0;
    GICR  |= (1<<INT1) ;//
//    sei();
}

SIGNAL( SIG_INTERRUPT1 )
{
    // disable int
    //cli();
    GICR  &= ~(1<<INT1) ;//
    touch_int_flag = 1;
}
